from enum import Enum
import os


class LCBInfo(Enum):
    DEFAULT_SLAVE_NO = 0x01
    LED_BASE_ADDRESS = 131
    BUZZER_BASE_ADDRESS = 151
    VIBRATION_STATUS_BASE_ADDRESS = 261
    BAY_DOOR_STATUS_BASE_ADDRESS = 281
    BUZZER_STATUS_BASE_ADDRESS = 301
    LED_STATUS_BASE_ADDRESS = 321
    COMMUNICATION_STATUS_BASE_ADDRESS = 351
    METHOD = "rtu"
    BAUDRATE = 9600
    PORT = "/dev/ttyS1"
    TIMEOUT = 1
    LCB_INFO_FILE = "/root/pod-sd/pod.ini"
    BAY_OFFSET = 1000
    POWER_PIN = 0
    DC_POWER_PIN = 2
    BAY_DOOR_PIN = 3
    BUZZER_FREQ_TYPES = ["start", "stop"]
    COLUMN_RANGE = 10
    LCB_OFF_TIMEOUT = 15
    DEFAULT_DOOR_COUNT = "7"
    DEVICE_TYPE = "device_type"
    CONFIG_SECTION_NAME = "lcb"
    FLASK_EXPOSED_METHODS = ["lcb_status", "get_serial_number", "get_firmware_version", "read_bay_door_status",
                             "lcb_status", "reset_lcb", "is_lcb_booted", "get_open_doors", "is_door_locked",
                             "power_on", "read_dc_power_status", "power_off"]


class LcdInfo(Enum):
    # Start: Pins configurations
    LCD_RS = 19
    LCD_E = 18
    LCD_D4 = 14
    LCD_D5 = 15
    LCD_D6 = 16
    LCD_D7 = 17
    LCD_BL = 1
    # End: Pins configurations

    LCD_WIDTH = 16
    LCD_CHR = True
    LCD_CMD = False

    LCD_LINE_1 = 0x80
    LCD_LINE_2 = 0xC0
    LCD_LINE_3 = 0x90
    LCD_LINE_4 = 0xD0

    E_PULSE = 0.0008

    LCD_REFRESH_TIMEOUT = 30
    FLASK_EXPOSED_METHODS = ["print_line", "fade", "clear", "start_blink", "stop_blink", "lcd_init_message"]


class KeypadInfo(Enum):
    END_KEY = "ENTER"
    CLEAR_KEY = "CLEAR"
    PRESS_KEY_TIMEOUT = 1
    LCD_HEIGHT = 4
    LCD_WIDTH = 16

    KEY_PRESSED_MAPPING = {
        0x81: "1",
        0x82: "5",
        0x83: "7",
        0x84: "#",
        0x8b: "2",
        0x8c: "*",
        0x8d: "8",
        0x8e: "+",
        0x95: "3",
        0x96: "-",
        0x97: "9",
        0x9f: "4",
        0xa0: "6",
        0xa1: "0"
    }

    KEY_RELEASED_MAPPING = {
        0x1: "1",
        0x2: "5",
        0x3: "7",
        0x4: "#",
        0xb: "2",
        0xc: "*",
        0xd: "8",
        0xe: "+",
        0x15: "3",
        0x16: "-",
        0x17: "9",
        0x1f: "4",
        0x20: "6",
        0x21: "0"
    }

    KEY_SLAVE_ID = 0x34
    CONFIG_REG = 0x01
    INT_STAT_REG = 0x02
    KEY_LCK_EVNT_REG = 0x03
    KEY_EVENT_REG = 0x04
    KP_GPIO1_REG = 0x1D
    KP_GPIO2_REG = 0x1E
    KP_GPIO3_REG = 0x1F

    KEY_INT = 37
    KEY_RESET_PIN = 43

    KEY_CODE_LENGTHS = [6]
    FLASK_EXPOSED_METHODS = ["keypad_init", "lock_keypad", "unlock_keypad"]