from main_includes import *

# order may matter so do not reorder
from podapi_utils import *

from main_modem import *
from main_networking import *
from main_lcb import *
from main_keypad import *
from main_display import *

# --------------------

door_inuse_state={1: "Free", 2: "Free", 3: "Free", 4: "Free"}


# **************************
# ****   MAIN DEMOFLOW  ****
# **************************

def open_demodoor_by_otp(otp_entered):
    Log.info("\topen_demodoor_by_otp: open_door_by_otp otp_entered=[" + str(otp_entered) + "]")
    display_update(line={1: "Verifying otp", 2: otp_entered, 3: "", 4: ""})

    door_number = -1

    # API CALL - should be injected here
    if (otp_entered == "111000"):

        door_number = 1
        if (door_inuse_state.get(door_number) != "InUse"):
            door_inuse_state.update({door_number: "InUse"})
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] opened for parcel drop")
        else:
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] door already InUse Invalid Otp")
            door_number = -1

    elif (otp_entered == "111001"):

        door_number = 1
        if (door_inuse_state.get(door_number) == "InUse"):
            door_inuse_state.update({door_number: "Free"})
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] opened for parcel pickup")
        else:
            door_number = -1
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] door empty Invalid otp")

    elif (otp_entered == "222000"):

        door_number = 2
        if (door_inuse_state.get(door_number) != "InUse"):
            door_inuse_state.update({door_number: "InUse"})
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] opened for parcel drop")
        else:
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] door already InUse Invalid Otp")
            door_number = -1

    elif (otp_entered == "222001"):

        door_number = 2
        if (door_inuse_state.get(door_number) == "InUse"):
            door_inuse_state.update({door_number: "Free"})
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] opened for parcel pickup")
        else:
            door_number = -1
            Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "] door empty Invalid otp")

    Log.info("\topen_demodoor_by_otp: door_number=[" + str(door_number) + "]")

    # Testing if the received door number is valid
    if (door_number is not None) and (door_number != -1):
        # A valid door number is passed for opening the lock
        open_demodoor_by_number(door_number)
    
    # Invalid Otp Entered - no valid associated door to open
    if (door_number is None) or (door_number == -1):
        Log.info("\topen_demodoor_by_otp: invalid door number")
        display_update(line={1: "", 2: "Invalid otp", 3: "", 4: "Try again"}, delay=USER_DISPLAY_VIEW_WAIT)
        # we might also want to inject some progressively increasing delay time
        # to make brute force attacks very frustratingly long

    Log.info("\topen_demodoor_by_otp: ended ...")
    return


def open_demodoor_by_number(door_number):
    Log.info("\topen_demodoor_by_number: open_door_by_Number open_door=[" + str(door_number) + "]")
    if not lcb_checkstatus():
        Log.info("\topen_demodoor_by_number: lcb not operational ... error")
        return

    # First open attempt to open the door
    lcb_open_lock(int(door_number))
    open_doors_list = lcb_get_opendoorslist()


    # Testing if successfully opened door ...
    if int(door_number) not in open_doors_list:
        # Second open attempt to make sure the door is fired open (**** ??? is this second open needed)
        lcb_open_lock(int(door_number))
        open_doors_list = lcb_get_opendoorslist()


    # Testing for successfull door open activity and Door Jam
    if int(door_number) not in open_doors_list:
        Log.info("\topen_demodoor_by_number: door jammed")
        display_update(line={3: "Door Jammed"}, delay=USER_DISPLAY_VIEW_WAIT)
    else:
        # remove the requested door from the list of open doors
        open_doors_list.remove(int(door_number))

        display_update(line={1: "", 2: "Opened", 3: "Door #:" + str(door_number)}, delay=USER_DISPLAY_VIEW_WAIT)

        display_update(line={1: "", 2: "Close", 3: "Door #:" + str(door_number)})

        # wait for the door to be closed
        while not lcb_get_isdoorlocked(door_number):
            Log.info("\topen_demodoor_by_number: waiting for door to be locked ...")
            time.sleep(0.3)
            pass

        Log.info("\topen_demodoor_by_number: door closed ...")
        display_update(line={1: "", 2: "", 3: "Thank you"}, delay=USER_DISPLAY_VIEW_WAIT)

    Log.info("\topen_demodoor_by_number: ended ...")
    return


def demoflow_main():
    Log.info("DEMOFLOW_MAIN: starting ...")

    door_inuse_state={1: "", 2: "", 3: "", 4: ""}

    while True:
        serialno = "" + str(net_current_serialno())
        display_clearall()
        display_update(line={1: "QikPod Demo", 2: "Enter Your OTP", 3: "", 4: ""})

        # small sleep between checks if next char was entered to add to opt entered
        time.sleep(0.3)

        # check is system is connected to internet and if not exit
        # Internet connectivity test and if there is no internet exit this module
        # status, _ = common_utils.ping()
        # if not status:
        #     Log.info("Podprod: no internet connectivity found ... so exiting ...")
        #     sys.exit()
        # return

        # need to periodically poll power status and update if we are on raw or UPS power
        lcb_update_powerstatus()

        Log.info("\tdemoflow_main: init screen")

        current_entered_otp = keypad_getotp(6, KEY_PRESS_WAIT_SECS)
        display_update(line={3: current_entered_otp})

        if current_entered_otp == "":
            # The otp was not yet valid or none was entered so loop again
            continue

        # if * Key is pressed goto Diag Screen
        elif current_entered_otp == "*":
            display_diagscreen()
            continue

        # if # Key is pressed debug exit
        elif current_entered_otp == "#":
            exit(1)

        # if CLEAR (-) key is pressed then clear the otp and go to init state
        elif current_entered_otp == "-":
            continue

        # Open the door only when OTP length is 6 exactly
        if len(current_entered_otp) == 6:
            Log.info("\tdemoflow_main: 6 digit OTP received ...")
            open_demodoor_by_otp(current_entered_otp)

        continue

    Log.info("\tDEMOFLOW_MAIN: ended ...")
    Log.info("\n")
    return


# --- CRITICAL GLOBAL VARIABLES DECLARATION SECTION BELOW ---


Log.info("MODULE - MAIN_DEMOFLOW: Started ...")

Log.info("\tMODULE - MAIN_DEMOFLOW: Completed ...")



