from main_includes import *

from podapi_utils import *

from main_networking import *
from main_modem import *
from main_lcb import *
from main_display import *
from main_keypad import *


# **************************
# **** MAIN DIAGNOSTICS ****
# **************************

def display_diagscreen():
    Log.info("display_diagscreen: started ...")
    display_clearall()

    mac = net_current_macaddress()
    serialno = net_current_serialno()
    ip = net_current_ipaddress()
    modemmode, strength, bars = modem_current_status()
    powermode = lcb_current_powerstatus()

    display_update(line={1: "Pod #:" + str(serialno),
                            2: "Mac:" + str(mac),
                            3: str(ip),
                            4: str(powermode) + ' ' + modemmode}, delay=USER_DISPLAY_VIEW_WAIT)

    display_clearall()
    Log.info("display_diagscreen: ended ...")
    return


def check_display():
    Log.info("check_display: start check_display")
    display_clearall()

    Log.info("check_display: start backlight test ...")
    display_togglebacklight()
    Log.info("check_display: stop backlight test ...")

    Log.info("check_display: start text line test ...")
    display_update(line={1: ".....QikPod....."})
    time.sleep(1.0)
    display_update(line={2: "abcdefghijklmnopq"})
    time.sleep(1.0)
    display_update(line={3: "rstuvwxyz1234567"})
    time.sleep(1.0)
    display_update(line={4: "890!@#$%^&**([];',."})
    time.sleep(1.0)

    Log.info("check_display: full display chars ...")
    display_update(line={1: "0123456789012345", 2: "1234567890123456", 3: "2345678901234567", 4: "3456789012345678"}, delay=USER_DISPLAY_VIEW_WAIT)

    Log.info("check_display: stop text line test ...")

    Log.info("check_display: clear display ...")
    display_clearall()
    display_update(line={1: "Completed", 2: "DISPLAY", 3: "Check"}, delay=USER_DISPLAY_VIEW_WAIT)

    display_clearall()
    Log.info("check_display: stop check_display")
    return


def check_keypad():
    Log.info("check_keypad: start check_keypad")
    display_clearall()

    display_update(line={1: "Press Any Key", 2: "CLEAR to Exit"})
    while True:
        pressed_key = keypad_getchar(KEY_PRESS_WAIT_SECS)
        Log.info("check_keypad: pressed_key=[" + str(pressed_key) + "]")
        if pressed_key == "":
            Log.info("check_keypad: no key entered ... looping ...")
            continue

        lcb_buzzerbeep()
        Log.info("check_keypad: check_keypad pressed_key=[" + str(pressed_key) + "]")
        display_update(line={3: pressed_key})

        time.sleep(1.0)

        if pressed_key == "-":
            Log.info("check_keypad: pressed_key=[" + str(pressed_key) + "] ... exiting loop ...")
            break

        continue

    Log.info("check_keypad: clear display ...")
    display_clearall()
    display_update(line={1: "Completed", 2: "KEYPAD", 3: "Check"}, delay=USER_DISPLAY_VIEW_WAIT)

    display_clearall()
    Log.info("check_keypad: stop check_keypad")
    return


def check_baydoor():
    Log.info("check_baydoor: start check_baydoor")
    display_clearall()

    display_update(line={1: "Baydoor Status", 2: "CLEAR to Exit"})
    while True:
        status = lcbobj.read_bay_door_status()
        if status is True:
            Log.info("check_baydoor: Bay Door is Open")
            display_update(line={3: "Open"})
        else:
            Log.info("check_baydoor: Bay Door is Closed")
            display_update(line={3: "Closed"})

        pressed_key = keypad_getchar(KEY_PRESS_WAIT_SECS)
        Log.info("check_baydoor: pressed_key=[" + str(pressed_key) + "]")
        if pressed_key is "-":
            Log.info("check_baydoor: pressed_key=[" + str(pressed_key) + "] ... exiting loop ...")
            break
        time.sleep(1.0)
        continue

    Log.info("check_baydoor: clear display ...")
    display_clearall()
    display_update(line={1: "Completed", 2: "BAYDOOR", 3: "Check"}, delay=USER_DISPLAY_VIEW_WAIT)

    display_clearall()
    Log.info("check_baydoor: stop check_baydoor")
    return


def check_opendoors():
    Log.info("check_opendoors: start check_opendoors")

    display_clearall()
    display_update(line={1: "Open Door #", 2: "CLEAR to Exit"})
    while True:
        pressed_key = keypad_getchar(KEY_PRESS_WAIT_SECS)
        if pressed_key is "-":
            Log.info("check_opendoors: pressed_key=[" + str(pressed_key) + "] ... exiting loop ...")
            break
        if pressed_key in ("*", "#", "+"):
            Log.info("check_opendoors: not implemeneted function entered ... ignoring ...")
            continue

        Log.info("check_opendoors: check_opendoors - open door=[" + pressed_key + "]")
        lcb_buzzerbeep()
        display_update(line={3: pressed_key})
        lcbobj.open_lock(pressed_key, "factory-test-key")
        time.sleep(1.0)

        continue

    display_clearall()
    display_update(line={1: "Completed", 2: "OPEN DOORS", 3: "Check"}, delay=USER_DISPLAY_VIEW_WAIT)

    Log.info("check_opendoors: clear display ...")
    display_clearall()
    Log.info("check_opendoors: stop check_opendoors")
    return


def check_closedoors():
    Log.info("check_closedoors: start check_closedoors")

    display_clearall()
    display_update(line={1: "Close Doors", 2: "CLEAR to Exit"})
    while True:
        Log.info("check_closedoors: waiting for menu input ...")
        pressed_key = keypad_getchar(KEY_PRESS_WAIT_SECS)
        Log.info("check_closedoors: pressed_key=[" + str(pressed_key) + "]")
        if pressed_key is "-":
            Log.info("check_closedoors: clear and exit requested ...")
            break
        if pressed_key in ("*", "#", "+"):
            Log.info("check_closedoors: not implemeneted function entered ... ignoring ...")
            continue

        doors = lcbobj.get_open_doors()
        if doors == []:
            Log.info("check_closedoors: check_closedoors - no open doors found")
            break

        Log.info("check_closedoors: doors open ... requested they be closed ....")
        display_update(line={1: "**CLOSE DOORS**", 3: "CLEAR to Exit"})

        time.sleep(1.0)
        continue

    display_clearall()
    display_update(line={1: "Completed", 2: "CLOSE DOORS", 3: "Check"}, delay=USER_DISPLAY_VIEW_WAIT)

    Log.info("check_closedoors: clear display ...")
    display_clearall()
    Log.info("check_closedoors: stop check_closedoors")
    return


def main_login_technician():
    display_clearall()

    display_update(line={1: "Enter Phone #"})
    user_phone = keypad_getotp(10, 45)
    Log.info("main_login_technician: user_phone=[" + str(user_phone) + "]")

    if len(user_phone) != 10:
        Log.info("main_login_technician: INVALID user_phone=[" + str(user_phone) + "]")
        return False

    Log.info("main_login_technician: podapi_generate_otp using user_phone=[" + str(user_phone) + "]")
    bret = podapi_generate_otp(user_phone)
    if bret is False:
        Log.info("main_login_technician: INVALID user_phone=[" + str(user_phone) + "]")
        return False

    display_update(line={1: "Enter OTP"})
    otp_text = keypad_getotp(4, 45)
    Log.info("main_login_technician: otp_text=[" + str(otp_text) + "]")

    if len(otp_text) != 4:
        Log.info("main_login_technician: INVALID otp_text=[" + str(otp_text) + "]")
        return False

    Log.info("main_login_technician: podapi_validate_otp using user_phone=[" + str(user_phone) + "], otp_text=[" + str(otp_text) + "]")
    bret = podapi_validate_otp(user_phone, otp_text)
    if bret is False:
        Log.info("main_login_technician: INVALID OTP user_phone=[" + str(user_phone) + "], otp_text=[" + str(otp_text) + "]")
        return False

    display_clearall()
    display_update(line={1: "LOGIN SUCCESS", 2: "Ph: " + user_phone, 3: "Otp: " + otp_text}, delay=USER_DISPLAY_VIEW_WAIT)

    display_clearall()
    Log.info("main_login_technician: INVALID OTP user_phone=[" + str(user_phone) + "], otp_text=[" + str(otp_text) + "]")
    return True


def main_diagnostics():
    Log.info("main_diagnostics: started ...")
    display_clearall()

    if not main_login_technician():
        return
    # if we get to this section it implies user phone and otp entered were valid

    serialno = "" + str(net_current_serialno())
    display_update(line={1: "Run Diagnostics", 2: "QP"+serialno, 3: "", 4: ""}, delay=USER_DISPLAY_VIEW_WAIT)

    check_display()
    check_keypad()
    check_opendoors()
    check_closedoors()
    check_baydoor()

    display_clearall()
    display_update(line={1: "Completed", 2: "All Diagnostics", 3: "Checks", 4: ""}, delay=USER_DISPLAY_VIEW_WAIT)

    display_clearall()
    Log.info("main_diagnostics: ended ...")
    return
