from main_includes import *

# order may matter so do not reorder
from podapi_utils import *

from main_modem import *
from main_networking import *
from main_lcb import *
from main_keypad import *
from main_display import *

# --------------------


def callback_lockerdoors(userdata):
    gflag_lockerdoors = True
    Log.info("callback_lockerdoors: invoked")
    return


# **************************
# **** MAIN PRODUCTION ****
# **************************

# work in progress
#def lcb_is_door_locked(door:None):
#


def wait_all_doors_closed():
    Log.info("\twait_all_doors_closed: started ...")
#    lcbobj.lcb_init()
    lcb_init()

    while True:
        if not lcb_checkstatus():
            Log.info("\twait_all_doors_closed: lcb not operationl ... exiting")
            return
        open_doors_list = lcb_get_opendoorslist()
        # There are doors open, looping until the open doors are closed
        for door in open_doors_list:
            door_isopen = lcb_get_isdoorlocked()
            # May need also test for LCB status here
            if not lcb_checkstatus():
                Log.info("\twait_all_doors_closed: lcb not operationl ... exit loop")
                break

            # Loop until all the doors is closed
            if not door_isopen:
                display_update(line={1: "", 2: "Please close", 3: "Door " + str(door), 4: ""})
                Log.info(door)
                while not lcb_get_isdoorlocked(door):
                    print("waiting for door to be closed")
                    pass
        continue
    Log.info("\twait_all_doors_closed: ended ...")
    return


def open_door_by_otp(otp_entered):
    Log.info("\topen_door_by_otp: open_door_by_otp otp_entered=[" + str(otp_entered) + "]")
    display_update(line={1: "Verifying otp", 2: otp_entered, 3: "", 4: ""})

    door_number = -1
    # API CALL - should be injected here
    if otp_entered == "111111":
        door_number = 1
    elif otp_entered == '222222':
        door_number = 2
    Log.info("\topen_door_by_otp: door_number=[" + str(door_number) + "]")

    # Testing if the received door number is valid
    if (door_number is not None) and (door_number != -1):
        # A valid door number is passed for opening the lock
        open_door_by_number(door_number)
    
    # Invalid Otp Entered - no valid associated door to open
    if (door_number is None) or (door_number == -1):
        Log.info("\topen_door_by_otp: invalid door number")
        display_update(line={1: "", 2: "Invalid otp", 3: "", 4: "Try again"}, delay=USER_DISPLAY_VIEW_WAIT)
        # we might also want to inject some progressively increasing delay time
        # to make brute force attacks very frustratingly long

    Log.info("\topen_door_by_otp: ended ...")
    return


def open_door_by_number(door_number):
    Log.info("\topen_door_by_number: open_door_by_Number open_door=[" + str(door_number) + "]")
    if not lcb_checkstatus():
        Log.info("\topen_door_by_number: lcb not operational ... error")
        return

    # First open attempt to open the door
    lcb_open_lock(int(door_number))
    open_doors_list = lcb_get_opendoorslist()


    # Testing if successfully opened door ...
    if int(door_number) not in open_doors_list:
        # Second open attempt to make sure the door is fired open (**** ??? is this second open needed)
        lcb_open_lock(int(door_number))
        open_doors_list = lcb_get_opendoorslist()


    # Testing for successfull door open activity and Door Jam
    if int(door_number) not in open_doors_list:
        Log.info("\topen_door_by_number: door jammed")
        display_update(line={3: "Door Jammed"}, delay=USER_DISPLAY_VIEW_WAIT)
    else:
        # remove the requested door from the list of open doors
        open_doors_list.remove(int(door_number))

        display_update(line={1: "", 2: "Opened", 3: "Door #:" + str(door_number)}, delay=USER_DISPLAY_VIEW_WAIT)

        display_update(line={1: "", 2: "Close", 3: "Door #:" + str(door_number)})

        # wait for the door to be closed
        while not lcb_get_isdoorlocked(door_number):
            Log.info("\topen_door_by_number: waiting for door to be locked ...")
            time.sleep(0.3)
            pass

        Log.info("\topen_door_by_number: door closed ...")
        display_update(line={1: "", 2: "", 3: "Thank you"}, delay=USER_DISPLAY_VIEW_WAIT)

    Log.info("\topen_door_by_number: ended ...")
    return


def main_production():
    Log.info("MAIN_PRODUCTION: starting ...")

    while True:
        serialno = "" + str(net_current_serialno())
        display_clearall()
        display_update(line={1: "QP"+serialno, 2: "Enter Your OTP", 3: "", 4: ""})

        # small sleep between checks if next char was entered to add to opt entered
        time.sleep(0.3)

        # check is system is connected to internet and if not exit
        # Internet connectivity test and if there is no internet exit this module
        # status, _ = common_utils.ping()
        # if not status:
        #     Log.info("Podprod: no internet connectivity found ... so exiting ...")
        #     sys.exit()
        # return

        # need to periodically poll power status and update if we are on raw or UPS power
        lcb_update_powerstatus()

        Log.info("\tmain_production: init screen")

        current_entered_otp = keypad_getotp(6, KEY_PRESS_WAIT_SECS)
        display_update(line={3: current_entered_otp})

        if current_entered_otp == "":
            # The otp was not yet valid or none was entered so loop again
            continue

        # if * Key is pressed goto Diag Screen
        elif current_entered_otp == "*":
            display_diagscreen()
            continue

        # if # Key is pressed debug exit
        elif current_entered_otp == "#":
            exit(1)

        # if CLEAR (-) key is pressed then clear the otp and go to init state
        elif current_entered_otp == "-":
            continue

        # Open the door only when OTP length is 6 exactly
        if len(current_entered_otp) == 6:
            Log.info("\tmain_production: 6 digit OTP received ...")
            open_door_by_otp(current_entered_otp)

        continue

    Log.info("\tMAIN_PRODUCTION: ended ...")
    Log.info("\n")
    return


# --- CRITICAL GLOBAL VARIABLES DECLARATION SECTION BELOW ---


Log.info("MODULE - MAIN_PRODUCTION: Started ...")

Log.info("\tMODULE - MAIN_PRODUCTION: setup locker door interrupt callback ...")
# Reset global flags which are signaled hig whenever lockerdoor & baydoor interrupt handlers are invoked
gflag_lockerdoors = True

door_intr = mraa.Gpio(LCB_DOOR)
door_intr.dir(mraa.DIR_IN)
# trap intr for baydoor open and close change states
door_intr.isr(mraa.EDGE_FALLING, callback_lockerdoors, None)

Log.info("\tMODULE - MAIN_PRODUCTION: Completed ...")
