import mraa
import Queue
import time
import logging
from constants.core_modules_info import KeypadInfo
from tools import reboot

LOGGER = logging.getLogger(__name__)


class KeyPad:

    instance = None

    @staticmethod
    def on_interrupt(_):
        if KeyPad.instance:
            KeyPad.instance.key_event()

    def __init__(self):
        self.stop_loop = False
        KeyPad.instance = self
        self.interrupt = mraa.Gpio(KeypadInfo.KEY_INT)
        self.i2c = mraa.I2c(0)
        self.key_reset = mraa.Gpio(KeypadInfo.KEY_RESET_PIN)
        self.key_queue = Queue.Queue()
        self.keypad_init()
        self.key_code = ""
        self.current_key = None
        self._keypad_lock = False
        self.last_key_entry = self.get_current_timestamp()
        self.last_lcd_refresh = self.get_current_timestamp()
        self.otp=""
        self.key_pressed = ""
        
    def keypad_init(self):
        self.key_reset.dir(mraa.DIR_OUT)
        self.key_reset.write(False)
        time.sleep(0.5)
        self.key_reset.write(True)
        time.sleep(0.5)
        self.i2c.address(KeypadInfo.KEY_SLAVE_ID)
        self.i2c.writeReg(KeypadInfo.CONFIG_REG, 0x01)
        self.i2c.writeReg(KeypadInfo.KEY_LCK_EVNT_REG, 0)
        self.i2c.writeReg(KeypadInfo.KP_GPIO1_REG, 0x0f)
        self.i2c.writeReg(KeypadInfo.KP_GPIO2_REG, 0x0f)
        self.i2c.writeReg(KeypadInfo.KP_GPIO3_REG, 0)
        self.i2c.writeReg(KeypadInfo.KEY_EVENT_REG, 0)
        self.interrupt.dir(mraa.DIR_IN)
        self.interrupt.isr(mraa.EDGE_FALLING, KeyPad.on_interrupt, None)

    @staticmethod
    def get_current_timestamp():
        return time.time()

    def key_event(self):
        key = self.i2c.readReg(KeypadInfo.KEY_EVENT_REG)
        current_time = self.get_current_timestamp()
        while True:
            data = self.i2c.readReg(KeypadInfo.KEY_EVENT_REG)
            # 255L is seen when the pins are shorted (touching or bad insulation)
            if data == 0:
                break
            if data == 255L:
                return
        self.i2c.writeReg(KeypadInfo.INT_STAT_REG, 0x01)
        if self._keypad_lock:
            self.clear_key_queue()
        else:
	        self.key_queue.put((key, current_time))
            

    def check_if_keypad_hung(self):
        if self.i2c.readReg(KeypadInfo.INT_STAT_REG) == 1:
            LOGGER.info("Reinitializing KeyPad...")
            self.keypad_init()

    def clear_key_queue(self):
        with self.key_queue.mutex:
            self.key_queue.queue.clear()

    def key_filter(self, key_entered):
        error = False
        if not key_entered:
            return None
        key, key_time = key_entered
        pressed_key = KeypadInfo.KEY_PRESSED_MAPPING.get(key)

        if pressed_key:
            if self.current_key is None or (key_time - self.current_key[1] > KeypadInfo.PRESS_KEY_TIMEOUT):
                self.current_key = (pressed_key, key_time)
            else:
                LOGGER.warn("Throwing never pressed key: " + pressed_key)
        released_key = KeypadInfo.KEY_RELEASED_MAPPING.get(key)
        if released_key:
            if self.current_key is None:
                LOGGER.warn("Throwing never released key: " + released_key)
            elif released_key != self.current_key[0]:
                LOGGER.warn("Throwing unmatched released key: " + released_key)
            else:
                self.current_key = None
                return released_key
        return None

