"""
Log Formatting file
"""
from logging import Logger
from logging.handlers import RotatingFileHandler
import logging
import sys
import threading
import os


LOG_FORMAT = '%(asctime)s %(levelname)-8s [%(processName)s] %(threadName)-20s %(name)-24s %(message)s'
LOG_FOLDER = os.getcwd()+'/logs/'
LOG_EXTN = '.log'

BACKUP_COUNT = 50

MAX_LOG_SIZE = 2 * 1024 * 1024


def initialize(log_file, cc_stdout, log_level=logging.INFO, use_thread_locking=True):
    if use_thread_locking:
        _configure_logger_to_use_locking(log_file)

    formatter = logging.Formatter(LOG_FORMAT)

    if cc_stdout:
        handler = logging.StreamHandler(sys.stdout)
        handler.setFormatter(formatter)
        logging.root.addHandler(handler)

    file_handler1 = RotatingFileHandler(log_file, maxBytes=MAX_LOG_SIZE, backupCount=BACKUP_COUNT)
    file_handler1.setFormatter(formatter)
    logging.root.addHandler(file_handler1)

    file_handler2 = RotatingFileHandler(
        LOG_FOLDER + "error" + LOG_EXTN, maxBytes=MAX_LOG_SIZE, backupCount=BACKUP_COUNT)
    file_handler2.setFormatter(formatter)
    file_handler2.setLevel(logging.ERROR)
    logging.root.addHandler(file_handler2)

    logging.root.setLevel(log_level)


def _configure_logger_to_use_locking(log_file):
    logging_lock_obj = threading.RLock()

    def call_handlers_with_locking(*args):
        logging_lock_obj.acquire()
        try:
            try:
                return Logger.orig_callHandlers(*args)  # @UndefinedVariable
            except (KeyboardInterrupt, SystemExit):
                raise
            except:
                pass
        finally:
            logging_lock_obj.release()

    Logger.orig_callHandlers = Logger.callHandlers
    Logger.callHandlers = call_handlers_with_locking
