# ************************************
# Pod API Python Helper Functions
# ************************************
import requests
import json

base_url = "https://podapi.qikpod.com:8989/"
# base_url = "http://127.0.0.1:8989/"


def call_api(request_type, endpoint, data=None, params=None):
    url = base_url + endpoint
    if request_type == "get":
        api_response = requests.get(url, params=params, verify=False)
    elif request_type == "put":
        api_response = requests.put(url, data=data, verify=False)
    elif request_type == "post":
        api_response = requests.post(url, data=data, verify=False)
    elif request_type == "patch":
        api_response = requests.patch(url, data=data, verify=False)
    elif request_type == "delete":
        api_response = requests.delete(url, params=params, verify=False)
    else:
        print("call_api - ERROR - unknown request_type")
        return ""

    return api_response.json()


def podapi_generate_otp(user_phone):
    if cloud_mode is True:
        request_data = {'user_phone': user_phone}
        response = call_api('get', 'generate_otp', params=request_data)
        print(response)  # for debugging
        breturn = response['success']
    else:
        # Do nothing if running in disconnected mode
        breturn = True
    return breturn


def podapi_validate_otp(user_phone, otp_text):
    if cloud_mode is True:
        request_data = {'user_phone': user_phone, 'otp_text': otp_text}
        response = call_api('get', 'validate_otp', params=request_data)
        print(response)  # for debugging
        breturn = response['success']
    else:
        # Do nothing if running in disconnected mode
        breturn = True
    return breturn


def podapi_match_otp_to_doorno(pod_id, otptext):
    otp_record_id = 0
    if cloud_mode is True:
        pod_data = {'pod_id': pod_id, 'otp_text': otptext}
        response = call_api('get', 'get_door', params=pod_data)
        print(response)  # for debugging
        '''
        response will look like    {
                                    "success": true,
                                    "door_number": 2,
                                    "otp_used": false,
                                    "record_id": 1
                                    }
        '''
        if response["success"] is True:
            # this id will be used when marking otp as used
            otp_record_id = response["record_id"]

            # check if the otp has been used
            if response["otp_used"] is False:
                doorno = response["door_number"]
            else:
                # if otp is used, return door number as -1
                doorno = -1

        # if no record was found for given pod_id and otp_text, then return doorno as -1
        else:
            doorno = -1

    else:

        # Return hardcoded values if running in disconnected mode

        if otptext == "111111":
            doorno = 1
        elif otptext == "222222":
            doorno = 2
        else:
            doorno = -1

    return {'door_number': doorno, 'otp_record_id': otp_record_id}


def podapi_mark_otp_as_used(otp_record_id):

    if cloud_mode is True:
        response = call_api('patch', 'set_otp_as_used/' + str(otp_record_id))
        print(response)  # for debugging
        breturn = response['success']

    else:
        # Do nothing if running in disconnected mode
        breturn = True

    return breturn


def podapi_savelog(log_eventtime="", log_level="INFO", log_type="pod", log_message="None", log_id=""):

    if cloud_mode is True:

        response = call_api('get', 'logs')
        print('GET logs response : \n %s' % response)

        # Sample of dict formulation
        # log_entry = {"log_eventtime": "2021-12-24T14:55:00", "log_level": "INFO",
        #                    "log_type": "pod", "log_message": "Test log", "log_id": "15457"}
        log_entry = {"log_eventtime": log_eventtime, "log_level": log_level, "log_type": log_type, "log_message": log_message, "log_id": log_id}

        request_data = json.dumps(log_entry)
        response = call_api('post', 'logs', request_data)
        print("podapi_savelog: request_data=[" + str(request_data) + "] response=[" + str(response) + "]")
        breturn = True

    else:

        # Do nothing if running in disconnected mode
        breturn = True

    return breturn


#
# podinfo_dict
# is specify any pending task which can later be updated to None after being completed
# specify pod health to whatever the pod status is actually
# set pod code version to whichever version is currently running on pod
# set version_download_url to whichever version needs to be run next
#


def podapi_pod_getinfo(pod_id):
    
    if cloud_mode is True:
        response = call_api('get', 'pods/' + str(pod_id))
        # print('GET pods response : \n %s' % response)
        podinfo_dict = response['records']

    else:

        # Do nothing if running in disconnected mode
        podinfo_dict = {}

    return podinfo_dict


def podapi_pod_updateinfo(pod_id, pod_dict):

    if cloud_mode is True:

        '''
        # sample data
        pod_dict = {"pod_health": "Yellow",
                    'pod_details': 'The details have been updated'}  # use valid field names!
        '''
        response = call_api('patch', 'pods/' + str(pod_id), data=json.dumps(pod_dict))
        breturn = response['success']

    else:

        # Do nothing if running in disconnected mode
        breturn = True

    return breturn

cloud_mode = True
