#!/bin/sh

LOG_FILE="$(pwd)/logs/monit.log"
log() {
	timeout -k 10 10 echo "$(date): $1" | tee -a $LOG_FILE
	logger -t wifi "$1"
}

disabled=$(uci get wireless.sta.disabled)

check_assoc() {
	ifname=`uci get wireless.sta.ifname`
	echo `iwconfig $ifname | awk 'FNR==2{print $6}'`
}

retries=0
while true
do
	if [ "$disabled" == "0" ]; then
		log "STA mode enabled"
		assoc=$(check_assoc)
		if [ "$assoc" != "Not-Associated" ]; then
			log "Connected, resetting retry count..."
			retries=0
		else
			log "Not connected, scanning now..."
			ssid=`uci get wireless.sta.ssid`
			key=`uci get wireless.sta.key`
			led=`uci get wireless.sta.led`
			ifname=`uci get wireless.sta.ifname`
			iwinfo "$ifname" scan | grep -wqF "$ssid"
			if [ $? == 0 ]; then
				log "Wifi ssid availble and still not connected, restarting wifi"
				retries=`expr $retries + 1`
				if [ $retries -le 10 ]; then
					/sbin/ap_client "ra0" "$ifname" "$ssid" "$key" "$led"
					sleep 60
				else
					log "Max retries reached to restart wifi with no success..."
				fi
			else
				log "SSID Not Found"
				apclient_pid=`pgrep ap_client`
				if [ $? == 0 ]; then
					log "SSID not available, so killing ap_client so it does not hog memory"
					kill $apclient_pid
				fi
			fi
		fi
	else
		log "Sta mode disabled"
	fi
	sleep 10
done
