"""
This file is used to execute all types of terminal commands
"""
# !/usr/bin/python
import commands
import logging
import os
import time
import traceback

import commentjson

# from manager.plugindef import PluginDef


LOGGER = logging.getLogger(__name__)

COMMANDS = {}
COMMAND_EXPIRY = 90


def execute(cmd, timeout, execution_time, pm):
    if execution_time is not None and (int(time.time()) - execution_time) > COMMAND_EXPIRY:
        return "Command expired: " + cmd
    custom_command = COMMANDS.get(cmd.split(" ")[0], None)
    if custom_command is not None:
        if custom_command.command_type == "PLUGIN_METHOD":
            try:
                method = pm.get_plugin_method(custom_command.plugin, custom_command.method, True)
                if len(cmd.split(" ")) > 1:
                    args = cmd.split(" ")[1]
                    args = dict(kv.split("=") for kv in args.split(" "))
                    return method(**args)
                else:
                    return method()
            except Exception:
                LOGGER.exception("Error in executing method {} of plugin {} through command channel".format(
                    custom_command.method, custom_command.plugin))
                return "Error!\n" + traceback.format_exc()
        elif custom_command.command_type == "SHELL":
            cmd = custom_command.command
            if timeout is None:
                timeout = custom_command.timeout
        else:
            return "Unsupported command"

    try:
        cmd_status, cmd_output = status_output(cmd, timeout)
        if cmd_status == 0:
            if cmd_output == "":
                cmd_output = "Empty Response"
        else:
            cmd_output = "Error!\nCode: {}\nOutput: {}".format(cmd_status, cmd_output)
    except Exception:
        cmd_output = "Error in executing command: {}".format(traceback.format_exc())

    return cmd_output


def status(command, timeout=30):
    return os.system("timeout -k 10 {} {}".format(timeout, command))


def output(command, timeout=30):
    return commands.getoutput("timeout -k 10 {} {}".format(timeout, command)).strip()


def status_output(command, timeout=30):
    return commands.getstatusoutput("timeout -k 10 {} {}".format(timeout, command))
